//----------------------------------------------------------------------
//							Timer Delay 
//				   Write By Mohammad mazarei @ 1390/6/17
//					Email: mohammad.mazarei@gmail.com
//----------------------------------------------------------------------
#include "pwm.h"
#include <at91sam7x256.h>
#include <lib_at91sam7x256.h>


static unsigned short FindClockConfiguration(
    unsigned int frequency,
    unsigned int mck)
{
    unsigned int divisors[11] = {1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024};
    unsigned char divisor = 0;
    unsigned int prescaler;
    //SANITY_CHECK(frequency < mck);
    // Find prescaler and divisor values
    prescaler = (mck / divisors[divisor]) / frequency;
    while ((prescaler > 255) && (divisor < 11)) {
        divisor++;
        prescaler = (mck / divisors[divisor]) / frequency;
    }
    // Return result
    if (divisor < 11) {
        return prescaler | (divisor << 8);
    }
    else {
        return 0;
    }
}


void PWMC_ConfigureClocks(unsigned int clka, unsigned int clkb, unsigned int mck)
{
    unsigned int mode = 0;
    unsigned int result;
    // Clock A
    if (clka != 0) {
        result = FindClockConfiguration(clka, mck);
        mode |= result;
    }
    // Clock B
    if (clkb != 0) {
        result = FindClockConfiguration(clkb, mck);
        mode |= (result << 16);
    }
    // Configure clocks
    AT91C_BASE_PWMC->PWMC_MR = mode;
}



void PWMC_SetPeriod(unsigned char channel, unsigned short period)
{
    // If channel is disabled, write to CPRD
    if ((AT91C_BASE_PWMC->PWMC_SR & (1 << channel)) == 0) {
        AT91C_BASE_PWMC->PWMC_CH[channel].PWMC_CPRDR = period;
    }
    else { // Otherwise use update register
        AT91C_BASE_PWMC->PWMC_CH[channel].PWMC_CMR |= AT91C_PWMC_CPD;
        AT91C_BASE_PWMC->PWMC_CH[channel].PWMC_CUPDR = period;
    }
}



void PWMC_SetDutyCycle(unsigned char channel, unsigned short duty)
{
    //SANITY_CHECK(duty <= AT91C_BASE_PWMC->PWMC_CH[channel].PWMC_CPRDR);

    // SAM7S errata

    // If channel is disabled, write to CDTY
    if ((AT91C_BASE_PWMC->PWMC_SR & (1 << channel)) == 0) {
        AT91C_BASE_PWMC->PWMC_CH[channel].PWMC_CDTYR = duty;
    }else {// Otherwise use update register
        AT91C_BASE_PWMC->PWMC_CH[channel].PWMC_CMR &= ~AT91C_PWMC_CPD;
        AT91C_BASE_PWMC->PWMC_CH[channel].PWMC_CUPDR = duty;
    }
}


void PWMC_ConfigureChannel(
    unsigned char channel,
    unsigned int prescaler,
    unsigned int alignment,
    unsigned int polarity)
{
    //SANITY_CHECK(prescaler < AT91C_PWMC_CPRE_MCKB);
    //SANITY_CHECK((alignment & ~AT91C_PWMC_CALG) == 0);
    //SANITY_CHECK((polarity & ~AT91C_PWMC_CPOL) == 0);

    // Disable channel
    AT91C_BASE_PWMC->PWMC_DIS = 1 << channel;
    // Configure channel
    AT91C_BASE_PWMC->PWMC_CH[channel].PWMC_CMR = prescaler | alignment | polarity;
}


void PWMC_EnableChannel(unsigned char channel)
{
   
	AT91C_BASE_PWMC->PWMC_ENA = 1 << channel;
}


void PWMC_DisableChannel(unsigned char channel)
{
    //AT91C_BASE_PMC->PMC_PCER &= ~(1 << AT91C_ID_PWMC);
	AT91C_BASE_PWMC->PWMC_DIS = 1 << channel;
}


void PWMC_EnableChannelIt(unsigned char channel)
{
	
    AT91C_BASE_PWMC->PWMC_IER = 1 << channel;
}


void PWMC_DisableChannelIt(unsigned char channel)
{
    AT91C_BASE_PWMC->PWMC_IDR = 1 << channel;
}
