//----------------------------------------------------------------------
//							Timer Delay 
//				   Write By Mohammad mazarei @ 1390/6/17
//					Email: mohammad.mazarei@gmail.com
//----------------------------------------------------------------------
#ifndef PWMC_H
#define PWMC_H

//------------------------------------------------------------------------------
//         Global functions
//------------------------------------------------------------------------------
/// PWMC PWM0 pin definition.
#define PIN_PWMC_PWM0   19
#define PORT_PWMC_PWM0 AT91C_BASE_PIOB

/// PWMC PWM1 pin definition.
#define PIN_PWMC_PWM1   20
#define PORT_PWMC_PWM1 AT91C_BASE_PIOB

/// PWMC PWM2 pin definition.
#define PIN_PWMC_PWM2   21
#define PORT_PWMC_PWM2 AT91C_BASE_PIOB

/// PWMC PWM3 pin definition.
#define PIN_PWMC_PWM3   22
#define PORT_PWMC_PWM3 AT91C_BASE_PIOB

#define PWMC_Configure_PIO(PWM_PORT,PWM_PIN)		PWM_PORT->PIO_PDR|=(1<<PWM_PIN);PWM_PORT->PIO_MDER|=(1<<PWM_PIN);PWM_PORT->PIO_ASR|=(1<<PWM_PIN)


//------------------------------------------------------------------------------
/// Configures PWM a channel with the given parameters.
/// The PWM controller must have been clocked in the PMC prior to calling this
/// function.
/// \param channel  Channel number.
/// \param prescaler  Channel prescaler.
/// \param alignment  Channel alignment.
/// \param polarity  Channel polarity.
//------------------------------------------------------------------------------
extern void PWMC_ConfigureChannel(
    unsigned char channel,
    unsigned int prescaler,
    unsigned int alignment,
    unsigned int polarity);

//------------------------------------------------------------------------------
/// Configures PWM clocks A & B to run at the given frequencies. This function
/// finds the best MCK divisor and prescaler values automatically.
/// \param clka  Desired clock A frequency (0 if not used).
/// \param clkb  Desired clock B frequency (0 if not used).
/// \param mck  Master clock frequency.
//------------------------------------------------------------------------------
extern void PWMC_ConfigureClocks
    (unsigned int clka,
     unsigned int clkb,
     unsigned int mck);

//------------------------------------------------------------------------------
/// Sets the period value used by a PWM channel. This function writes directly
/// to the CPRD register if the channel is disabled; otherwise, it uses the
/// update register CUPD.
/// \param channel  Channel number.
/// \param period  Period value.
//------------------------------------------------------------------------------
extern void PWMC_SetPeriod(unsigned char channel, unsigned short period);

//------------------------------------------------------------------------------
/// Sets the duty cycle used by a PWM channel. This function writes directly to
/// the CDTY register if the channel is disabled; otherwise it uses the
/// update register CUPD.
/// Note that the duty cycle must always be inferior or equal to the channel
/// period.
/// \param channel  Channel number.
/// \param duty  Duty cycle value.
//------------------------------------------------------------------------------
extern void PWMC_SetDutyCycle(unsigned char channel, unsigned short duty);

//------------------------------------------------------------------------------
/// Enables the given PWM channel. This does NOT enable the corresponding pin;
/// this must be done in the user code.
/// \param channel  Channel number.
//------------------------------------------------------------------------------
extern void PWMC_EnableChannel(unsigned char channel);

//------------------------------------------------------------------------------
/// Disables the given PWM channel.
/// \param channel  Channel number.
//------------------------------------------------------------------------------
extern void PWMC_DisableChannel(unsigned char channel);

//------------------------------------------------------------------------------
/// Enables the period interrupt for the given PWM channel.
/// \param channel  Channel number.
//------------------------------------------------------------------------------
extern void PWMC_EnableChannelIt(unsigned char channel);

//------------------------------------------------------------------------------
/// Disables the period interrupt for the given PWM channel.
/// \param channel  Channel number.
//------------------------------------------------------------------------------
extern void PWMC_DisableChannelIt(unsigned char channel);


#endif //#ifndef PWMC_H
